==> moonlight-1.20-2.16.14-fabric-sources.jar: net/mehvahdjukaar/moonlight/core/mixins/ReloadableClientResourcesMixin.java <==
package net.mehvahdjukaar.moonlight.core.mixins;

import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;
import net.mehvahdjukaar.moonlight.api.platform.PlatHelper;
import net.mehvahdjukaar.moonlight.core.misc.FilteredResManager;
import net.mehvahdjukaar.moonlight.core.misc.ReloadInstanceWrapper;
import net.minecraft.class_2960;
import net.minecraft.class_3264;
import net.minecraft.class_3300;
import net.minecraft.class_3302;
import net.minecraft.class_3304;
import net.minecraft.class_3902;
import net.minecraft.class_4011;
import net.minecraft.class_6860;
import net.minecraft.server.packs.resources.*;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.injection.At;

import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;

@Mixin(class_3304.class)
public abstract class ReloadableClientResourcesMixin {

    @Shadow
    @Final
    public class_3264 type;

    @Shadow
    private class_6860 resources;

    //should fire right before add reload listener, before packs are reloaded and listeners called
    @WrapOperation(method = "createReload", at = @At(value = "INVOKE", target = "Lnet/minecraft/server/packs/resources/SimpleReloadInstance;create(Lnet/minecraft/server/packs/resources/ResourceManager;Ljava/util/List;Ljava/util/concurrent/Executor;Ljava/util/concurrent/Executor;Ljava/util/concurrent/CompletableFuture;Z)Lnet/minecraft/server/packs/resources/ReloadInstance;"))
    private class_4011 moonlight$clientDynamicPackEarlyReload(class_3300 resourceManager,
                                                                  List<class_3302> listeners,
                                                                  Executor backgroundExecutor, Executor gameExecutor,
                                                                  CompletableFuture<class_3902> alsoWaitedFor, boolean profiled,
                                                                  Operation<class_4011> original) {
        //fires on world load or on /reload
        //token to assure that modded resources are included
        if (!(this.resources instanceof FilteredResManager) &&
                this.resources.method_14486(new class_2960("moonlight:moonlight/token.json")).isPresent()) { //this assumes that it includes all pack including all mod assets
            //one would think that this would be fool proof. Well check again, some mod like to re create this resource manager during block load! All modded resources included aswell
            //so to be EXTRA safe we check if registry phase is over
            if (!PlatHelper.isInitializing()) {
                return ReloadInstanceWrapper.wrap(() -> original.call(resourceManager, listeners,
                                backgroundExecutor, gameExecutor, alsoWaitedFor, profiled),
                        type, this.resources, backgroundExecutor);
            }
        }
        return original.call(resourceManager, listeners, backgroundExecutor, gameExecutor, alsoWaitedFor, profiled);
    }
}

==> moonlight-1.21-2.25.0-fabric-sources.jar: net/mehvahdjukaar/moonlight/core/mixins/ReloadableClientResourcesMixin.java <==
package net.mehvahdjukaar.moonlight.core.mixins;

import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;
import net.mehvahdjukaar.moonlight.api.platform.PlatHelper;
import net.mehvahdjukaar.moonlight.core.misc.FilteredResManager;
import net.mehvahdjukaar.moonlight.core.misc.ReloadInstanceWrapper;
import net.minecraft.class_2960;
import net.minecraft.class_3262;
import net.minecraft.class_3264;
import net.minecraft.class_3300;
import net.minecraft.class_3302;
import net.minecraft.class_3304;
import net.minecraft.class_3902;
import net.minecraft.class_4011;
import net.minecraft.class_6860;
import net.minecraft.server.packs.resources.*;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.injection.At;

import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import java.util.stream.Stream;

@Mixin(class_3304.class)
public abstract class ReloadableClientResourcesMixin {

    @Shadow
    @Final
    public class_3264 type;

    @Shadow
    private class_6860 resources;

    @Shadow public abstract Stream<class_3262> listPacks();

    //should fire right before add reload listener, before packs are reloaded and listeners called
    @WrapOperation(method = "createReload", at = @At(value = "INVOKE", target = "Lnet/minecraft/server/packs/resources/SimpleReloadInstance;create(Lnet/minecraft/server/packs/resources/ResourceManager;Ljava/util/List;Ljava/util/concurrent/Executor;Ljava/util/concurrent/Executor;Ljava/util/concurrent/CompletableFuture;Z)Lnet/minecraft/server/packs/resources/ReloadInstance;"))
    private class_4011 moonlight$clientDynamicPackEarlyReload(class_3300 resourceManager,
                                                                  List<class_3302> listeners,
                                                                  Executor backgroundExecutor, Executor gameExecutor,
                                                                  CompletableFuture<class_3902> alsoWaitedFor, boolean profiled,
                                                                  Operation<class_4011> original) {
        //fires on world load or on /reload
        //token to assure that modded resources are included
        if (!(this.resources instanceof FilteredResManager) &&
                this.resources.method_14486(class_2960.method_60654("moonlight:moonlight/token.json")).isPresent()) { //this assumes that it includes all pack including all mod assets
            //one would think that this would be fool proof. Well check again, some mod like to re create this resource manager during block load! All modded resources included aswell
            //so to be EXTRA safe we check if registry phase is over
            if (!PlatHelper.isInitializing()) {
                return ReloadInstanceWrapper.wrap(() -> original.call(resourceManager,
                               List.copyOf(listeners), //TODO: maybe change. some mod are adding stuff to these incorrectly after the reload instance is created. Doing this so we dont get reports redirected here aswell
                                backgroundExecutor, gameExecutor, alsoWaitedFor, profiled),
                        type, this.resources, backgroundExecutor,
                        listPacks().toList());
            }
        }
        return original.call(resourceManager, listeners, backgroundExecutor, gameExecutor, alsoWaitedFor, profiled);
    }
}
